#This source code is able to freely use for Blender and users.

bl_info = {
    "name": "Add point of intersection",
    "author": "TMO",
    "version": (0, 1, 0),
    "blender": (2, 6, 0),
    "location": "View3D > Add > Mesh > Add point of intersection",
    "description": "Add point of intersection",
    "warning": "",
    "wiki_url": "http://www.sousakuba.com/blender/index.html",
    "tracker_url": "",
    "category": "3D View"}

import bpy
from bpy.props import *
from bpy_extras import object_utils
import mathutils

class MSG():
    msg_type = ''
    msg = ''

class MessageBox(bpy.types.Operator, MSG):
    bl_idname = 'va.msg_box'
    bl_label = ''

    def draw(self, context):
        if self.msg_type == 'ERROR':
            self.t = 'ERROR'
        else:
            self.t = 'NONE'

        layout = self.layout
        row = layout.split(0.25)
        row.label(self.msg_type, icon = self.t)
        row.label(self.msg)

    def execute(self, context):
        return {'FINISHED'}

    def invoke(self, context, event):
        return context.window_manager.invoke_popup(self)

def vlen(v):
    return pow( v.x*v.x + v.y*v.y + v.z*v.z, 0.5 )

def distNP(a,b):
    return (b.x-a.x)*(b.x-a.x)+(b.y-a.y)*(b.y-a.y)+(b.z-a.z)*(b.z-a.z)

def normal_vector(v):
    v_len = vlen(v);
    v.x /= v_len;
    v.y /= v_len;
    v.z /= v_len;

def zoom(v, val):
    v.x *= val;
    v.y *= val;
    v.z *= val;

def cross_product(l,r):
    return mathutils.Vector( (l.y * r.z) - (l.z * r.y), (l.z * r.x) - (l.x * r.z), (l.x * r.y) - (l.y * r.x) )

def dot_product(l,r):
    return l.x * r.x + l.y * r.y + l.z * r.z

def chk_range(a,b,val):
    ab    = mathutils.Vector( ( b.x - a.x, b.y - a.y, b.z - a.z ) )
    a_val = mathutils.Vector( ( val.x - a.x, val.y - a.y, val.z - a.z ) )
    b_val = mathutils.Vector( ( val.x - b.x, val.y - b.y, val.z - b.z ) )

    len_ab = vlen(ab)
    len_a_val = vlen(a_val)
    len_b_val = vlen(b_val)

    min_val = min( ( len_a_val ,len_b_val ) )

    if min_val < len_ab:
        return True

    return False;

def create_intersect( a,b,c,d ):

    ab = mathutils.Vector( ( b.x - a.x, b.y - a.y, b.z - a.z ) )
    cd = mathutils.Vector( ( d.x - c.x, d.y - c.y, d.z - c.z ) )

    t0 = mathutils.Vector( ( b.x - a.x, b.y - a.y, b.z - a.z ) )
    t1 = mathutils.Vector( ( d.x - c.x, d.y - c.y, d.z - c.z ) )

    normal_vector(t0)
    normal_vector(t1)

    spd = dot_product(t0,t1)
    det = spd*spd - 1.0

    if det==0.0:
        return None

    ac = mathutils.Vector( ( c.x - a.x, c.y - a.y, c.z - a.z ) )

    u0 = ( spd * dot_product(ac,t1) - dot_product(ac, t0) ) / det
    u1 = ( dot_product(ac,t1) - spd * dot_product(ac, t0) ) / det

    #print("u0:%f \n", u0)
    #print("u1:%f \n", u1)

    #on ab
    ret1 = mathutils.Vector( ( a.x + t0.x * u0, a.y + t0.y * u0, a.z + t0.z * u0 ) )

    #on bc
    ret2 = mathutils.Vector( ( c.x + t1.x * u1, c.y + t1.y * u1, c.z + t1.z * u1 ) )

    ret = []

    ret.append(ret1)

    #if distNP(ret1, ret2) > 0.0000001:
    ret.append(ret2)

    return ret

class main(bpy.types.Operator):
    bl_idname="mesh.add_point_of_intersection"
    bl_label="Add point of intersection"
    bl_options = {'REGISTER', 'UNDO'}

    def draw(self, context):
        return

    def execute(self, context):

        #need for update select
        bpy.ops.object.editmode_toggle()
        bpy.ops.object.editmode_toggle()

        mesh = bpy.context.active_object.data

        e_sel = [e for e in mesh.edges if e.select]

        if len( e_sel ) is not 2:
            MSG.msg_type = 'ERROR'
            MSG.msg = 'Please select 2 edges.'
            bpy.ops.va.msg_box('INVOKE_DEFAULT')
            return {'FINISHED'}

        A = mesh.vertices[ e_sel[0].vertices[0] ].co;
        B = mesh.vertices[ e_sel[0].vertices[1] ].co;
        C = mesh.vertices[ e_sel[1].vertices[0] ].co;
        D = mesh.vertices[ e_sel[1].vertices[1] ].co;

        ret = create_intersect(A,B,C,D)

        if ret==None:
            MSG.msg_type = 'ERROR'
            MSG.msg = 'Cannot calculate intersection.'
            bpy.ops.va.msg_box('INVOKE_DEFAULT')
            return {'FINISHED'}

        ret2 = []
        if chk_range( A,B,ret[0] ):
            ret2.append(ret[0])
        if chk_range( C,D,ret[1] ):
            if len(ret) is 1:
                if distNP(ret[0], ret[1]) > 0.0000001:
                    ret2.append(ret[1])
            else:
                ret2.append(ret[1])

        ret = ret2

        if len(ret) is 0:
            MSG.msg_type = 'ERROR'
            MSG.msg = 'Intersection is out of range.'
            bpy.ops.va.msg_box('INVOKE_DEFAULT')
            return {'FINISHED'}
    #----------------------------------------------------
        cursor = bpy.context.scene.cursor_location

        verts = []

        verts.append( [ ret[0].x-cursor[0], ret[0].y-cursor[1], ret[0].z-cursor[2] ] )

        if len(ret) is 2:
            verts.append( [ ret[1].x-cursor[0], ret[1].y-cursor[1], ret[1].z-cursor[2] ] )

        n_mesh = bpy.data.meshes.new("mesh")
        n_mesh.from_pydata(verts, [], [])
        n_mesh.update()
        object_utils.object_data_add(context, n_mesh, operator=None)



        return {'FINISHED'}

def menu_func(self, context):
    self.layout.operator(main.bl_idname, main.bl_label, icon="PLUGIN")

def register():
    bpy.utils.register_module(__name__)

    bpy.types.INFO_MT_mesh_add.append(menu_func)

def unregister():
    bpy.utils.unregister_module(__name__)

    bpy.types.INFO_MT_mesh_add.remove(menu_func)

if __name__ == "__main__":
    register()
